package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.ApiInfo;
import be.iminds.ilabt.jfed.lowlevel.api.OCCI;
import be.iminds.ilabt.jfed.lowlevel.api.RestApi;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.*;
import be.iminds.ilabt.jfed.lowlevel.connection.HttpConnection;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JavaFXLogger;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 11:39
 */
public final class GetOcciNetworksForLocationTask extends Task {
    private final Server auth;
    private final String locationId;

    private final JavaFXLogger logger;
    private final JFedPreferences jFedPreferences;
    private final GeniUserProvider geniUserProvider;
    private final JFedConnectionProvider connectionProvider;

    public GetOcciNetworksForLocationTask(Server auth, String locationId, JavaFXLogger logger,
                                          JFedPreferences jFedPreferences, GeniUserProvider geniUserProvider,
                                          JFedConnectionProvider connectionProvider) {
        super("Fetch OCCI networks for location " + locationId);
        this.auth = auth;
        this.locationId = locationId;
        this.logger = logger;
        this.jFedPreferences = jFedPreferences;
        this.geniUserProvider = geniUserProvider;
        this.connectionProvider = connectionProvider;
    }

    @Override
    public void doTask(TaskExecution taskExecution) throws JFedException, InterruptedException {
        OCCI occiApi = new OCCI(logger, jFedPreferences);

        GeniUser user = geniUserProvider.getLoggedInGeniUser();
        HttpConnection con = (HttpConnection)
                connectionProvider.getConnectionByAuthority(user, auth, new ApiInfo.Api(ApiInfo.ApiName.BONFIRE_OCCI, 1));
        if (con == null)
            throw new RuntimeException("No connection can be created. Perhaps the authority \"" + auth.getDefaultComponentManagerUrn() + "\" does not have an OCCI interface?");

        RestApi.RestReply<String> reply = occiApi.getLocationNetworks(con, locationId);

        if (!reply.getGeniResponseCode().isSuccess())
            throw new JFedException("OCCI Reply is not success. request URL=" +
                    (reply.getHttpCallDetails() == null ? "null" : reply.getHttpCallDetails().getServerUrl())
                    + "\n\nHTTP reply=" +
                    (reply.getHttpCallDetails() == null ? "null" :
                            (reply.getHttpCallDetails().getResultHttpStatusLine() + "\n" +
                                    reply.getHttpCallDetails().getResultHttpHeaders() + "\n\n" +
                                    reply.getHttpCallDetails().getResultHttpContent())
                    )
            );
    }
}
